@@ -22,8 +22,8 @@ OC.L10N.register(
     "Keyboard shortcuts" : "কী-বোর্ড শর্টকাট",
     "Rename" : "পূনঃনামকরণ",
     "Delete" : "মুছে",
-    "Previous" : "পূর্ববর্তী",
     "by" : "কর্তৃক",
+    "Folder" : "ফোল্ডার",
     "right" : "ডান",
     "left" : "বাম",
     "Refresh" : "নবোদ্যম"
