@@ -14,6 +14,7 @@ OC.L10N.register(
     "Folder exists already!" : "Dossier ja existe!",
     "Credentials" : "Datos de authentication",
     "Subscribe" : "Subscribe",
+    "Show all articles" : "Monstrar tote articulos",
     "Move" : "Mover",
     "Share" : "Compartir",
     "All articles" : "Tote articulos",
@@ -23,12 +24,9 @@ OC.L10N.register(
     "Documentation" : "Documentation",
     "Keyboard shortcuts" : "Combinationes de claves",
     "Disable mark read through scrolling" : "Disactivar le marcation como legite per rolamento",
-    "Show all articles" : "Monstrar tote articulos",
     "Reverse ordering (oldest on top)" : "Ordination reverse (le plus ancian in le topo)",
     "Rename" : "Renominar",
     "Delete" : "Deler",
-    "Previous" : "Previe",
-    "Next" : "Proxime",
     "Toggle star article" : "Alternar stato de favorito del articulo",
     "by" : "per",
     "Play audio" : "Reproducer audio",
@@ -36,6 +34,7 @@ OC.L10N.register(
     "Download video" : "Discargar video",
     "Open website" : "Aperir sito web",
     "Keep article unread" : "Mantener articulo como non legite",
+    "Folder" : "Dossier",
     "Keyboard shortcut" : "Combination de claves",
     "Jump to next article" : "Saltar al articulo sequente",
     "Jump to previous article" : "Saltar al articulo previe",
