@@ -1,8 +1,8 @@
 OC.L10N.register(
     "news",
     {
-    "Download" : "Download",
-    "Close" : "Close",
+    "Download" : "Yuklab olish",
+    "Close" : "Yopish",
     "Folder name" : "Folder name",
     "Username" : "Username",
     "Password" : "Password",
@@ -11,13 +11,15 @@ OC.L10N.register(
     "Subscribe" : "Obuna boʻlish",
     "Move" : "Move",
     "Share" : "Ulashish",
+    "Default" : "Standart",
     "Off" : "Oʻchirilgan",
-    "Settings" : "Settings",
+    "Settings" : "Sozlamalar",
     "Documentation" : "Hujjatlar",
-    "Rename" : "Rename",
-    "Delete" : "Delete",
-    "Not available" : "Mavjud emas",
+    "Keyboard shortcuts" : "Klaviatura yorliqlari",
+    "Rename" : "Nomini o'zgartirish",
+    "Delete" : "O'chirish",
     "Title" : "Sarlavha",
+    "Not available" : "Mavjud emas",
     "right" : "right",
     "left" : "left",
     "Refresh" : "Refresh"
